@@ -8,6 +8,7 @@ __docformat__ = 'restructuredtext en'
 import numbers
 from collections import defaultdict
 from datetime import datetime, timedelta
+from typing import Iterable
 
 from calibre.ebooks.metadata import author_to_author_sort
 from calibre.utils.date import UNDEFINED_DATE, parse_date, utc_tz
@@ -52,21 +53,27 @@ null = object()
 
 class Table:
 
+    supports_notes = False
+
     def __init__(self, name, metadata, link_table=None):
         self.name, self.metadata = name, metadata
         self.sort_alpha = metadata.get('is_multiple', False) and metadata.get('display', {}).get('sort_alpha', False)
+        dt = metadata['datatype']
 
         # self.unserialize() maps values from the db to python objects
         self.unserialize = {
             'datetime': c_parse,
             'bool': bool
-        }.get(metadata['datatype'], None)
+        }.get(dt)
+        self.serialize = None
         if name == 'authors':
             # Legacy
             self.unserialize = lambda x: x.replace('|', ',') if x else ''
-
+            self.serialize = lambda x: x.replace(',', '|')
         self.link_table = (link_table if link_table else
                 'books_%s_link'%self.metadata['table'])
+        if self.supports_notes and dt == 'rating':  # custom ratings table
+            self.supports_notes = False
 
     def remove_books(self, book_ids, db):
         return set()
@@ -199,6 +206,7 @@ class ManyToOneTable(Table):
     '''
 
     table_type = MANY_ONE
+    supports_notes = True
 
     def read(self, db):
         self.id_map = {}
@@ -245,16 +253,42 @@ class ManyToOneTable(Table):
             if len(v) > 1:
                 main_id = min(v)
                 v.discard(main_id)
+                item_map = {}
                 for item_id in v:
-                    self.id_map.pop(item_id, None)
+                    val = self.id_map.pop(item_id, null)
+                    if val is not null:
+                        item_map[item_id] = val
                     books = self.col_book_map.pop(item_id, set())
                     for book_id in books:
                         self.book_col_map[book_id] = main_id
                 db.executemany('UPDATE {0} SET {1}=? WHERE {1}=?'.format(
                     self.link_table, self.metadata['link_column']),
                     tuple((main_id, x) for x in v))
-                db.executemany('DELETE FROM {} WHERE id=?'.format(self.metadata['table']),
-                    tuple((x,) for x in v))
+                db.delete_category_items(self.name, self.metadata['table'], item_map)
+
+    def item_ids_for_names(self, db, item_names: Iterable[str], case_sensitive: bool = False) -> dict[str, int]:
+        item_names = tuple(item_names)
+        if case_sensitive:
+            colname = self.metadata['column']
+            serialized_names = tuple(map(self.serialize, item_names)) if self.serialize else item_names
+            if len(item_names) == 1:
+                iid = db.get(f'SELECT id FROM {self.metadata["table"]} WHERE {colname} = ?', ((serialized_names[0],)), all=False)
+                return {item_names[0]: iid}
+            inq = ('?,' * len(item_names))[:-1]
+            ans = dict.fromkeys(item_names)
+            res = db.get(f'SELECT {colname}, id FROM {self.metadata["table"]} WHERE {colname} IN ({inq})', serialized_names)
+            if self.unserialize:
+                res = ((self.unserialize(name), iid) for name, iid in res)
+            ans.update(res)
+            return ans
+        if len(item_names) == 1:
+            q = icu_lower(item_names[0])
+            for iid, name in self.id_map.items():
+                if icu_lower(name) == q:
+                    return {item_names[0]: iid}
+            return {item_names[0]: iid}
+        rmap = {icu_lower(v) if isinstance(v, str) else v:k for k, v in self.id_map.items()}
+        return {name: rmap.get(icu_lower(name) if isinstance(name, str) else name, None) for name in item_names}
 
     def remove_books(self, book_ids, db):
         clean = set()
@@ -308,22 +342,26 @@ class ManyToOneTable(Table):
                 affected_books |= self.remove_items(items_to_process_normally, db)
             return affected_books
 
+        item_map = {}
         for item_id in item_ids:
             val = self.id_map.pop(item_id, null)
             if val is null:
                 continue
+            item_map[item_id] = val
             book_ids = self.col_book_map.pop(item_id, set())
             for book_id in book_ids:
                 self.book_col_map.pop(book_id, None)
             affected_books.update(book_ids)
-        item_ids = tuple((x,) for x in item_ids)
-        db.executemany('DELETE FROM {} WHERE {}=?'.format(self.link_table, self.metadata['link_column']), item_ids)
-        db.executemany('DELETE FROM {} WHERE id=?'.format(self.metadata['table']), item_ids)
+        db.delete_category_items(self.name, self.metadata['table'], item_map, self.link_table, self.metadata['link_column'])
         return affected_books
 
     def rename_item(self, item_id, new_name, db):
-        rmap = {icu_lower(v):k for k, v in iteritems(self.id_map)}
-        existing_item = rmap.get(icu_lower(new_name), None)
+        existing_item = None
+        q = icu_lower(new_name)
+        for q_id, q_val in self.id_map.items():
+            if icu_lower(q_val) == q:
+                existing_item = q_id
+                break
         table, col, lcol = self.metadata['table'], self.metadata['column'], self.metadata['link_column']
         affected_books = self.col_book_map.get(item_id, set())
         new_id = item_id
@@ -339,11 +377,7 @@ class ManyToOneTable(Table):
             for book_id in books:
                 self.book_col_map[book_id] = existing_item
             self.col_book_map[existing_item].update(books)
-            # For custom series this means that the series index can
-            # potentially have duplicates/be incorrect, but there is no way to
-            # handle that in this context.
-            db.execute('UPDATE {0} SET {1}=? WHERE {1}=?; DELETE FROM {2} WHERE id=?'.format(
-                self.link_table, lcol, table), (existing_item, item_id, item_id))
+            db.rename_category_item(self.name, table, self.link_table, lcol, item_id, existing_item, self.id_map[new_id])
         return affected_books, new_id
 
     def set_links(self, link_map, db):
@@ -358,6 +392,8 @@ class ManyToOneTable(Table):
 
 class RatingTable(ManyToOneTable):
 
+    supports_notes = False
+
     def read_id_maps(self, db):
         ManyToOneTable.read_id_maps(self, db)
         # Ensure there are no records with rating=0 in the table. These should
@@ -405,7 +441,7 @@ class ManyToManyTable(ManyToOneTable):
                 self.link_table, self.metadata['link_column']), tuple((x,) for x in extra_item_ids))
 
     def remove_books(self, book_ids, db):
-        clean = set()
+        clean = {}
         for book_id in book_ids:
             item_ids = self.book_col_map.pop(book_id, ())
             for item_id in item_ids:
@@ -417,13 +453,12 @@ class ManyToManyTable(ManyToOneTable):
                 else:
                     if not self.col_book_map[item_id]:
                         del self.col_book_map[item_id]
-                        if self.id_map.pop(item_id, null) is not null:
-                            clean.add(item_id)
+                        val = self.id_map.pop(item_id, null)
+                        if val is not null:
+                            clean[item_id] = val
         if clean and self.do_clean_on_remove:
-            db.executemany(
-                'DELETE FROM {} WHERE id=?'.format(self.metadata['table']),
-                [(x,) for x in clean])
-        return clean
+            db.delete_category_items(self.name, self.metadata['table'], clean)
+        return set(clean)
 
     def remove_items(self, item_ids, db, restrict_to_book_ids=None):
         affected_books = set()
@@ -456,22 +491,26 @@ class ManyToManyTable(ManyToOneTable):
                 affected_books |= self.remove_items(items_to_process_normally, db)
             return affected_books
 
+        item_map = {}
         for item_id in item_ids:
             val = self.id_map.pop(item_id, null)
             if val is null:
                 continue
+            item_map[item_id] = val
             book_ids = self.col_book_map.pop(item_id, set())
             for book_id in book_ids:
                 self.book_col_map[book_id] = tuple(x for x in self.book_col_map.get(book_id, ()) if x != item_id)
             affected_books.update(book_ids)
-        item_ids = tuple((x,) for x in item_ids)
-        db.executemany('DELETE FROM {} WHERE {}=?'.format(self.link_table, self.metadata['link_column']), item_ids)
-        db.executemany('DELETE FROM {} WHERE id=?'.format(self.metadata['table']), item_ids)
+        db.delete_category_items(self.name, self.metadata['table'], item_map, self.link_table, self.metadata['link_column'])
         return affected_books
 
     def rename_item(self, item_id, new_name, db):
-        rmap = {icu_lower(v):k for k, v in iteritems(self.id_map)}
-        existing_item = rmap.get(icu_lower(new_name), None)
+        existing_item = None
+        q = icu_lower(new_name)
+        for q_id, q_val in self.id_map.items():
+            if icu_lower(q_val) == q:
+                existing_item = q_id
+                break
         table, col, lcol = self.metadata['table'], self.metadata['column'], self.metadata['link_column']
         affected_books = self.col_book_map.get(item_id, set())
         new_id = item_id
@@ -492,8 +531,7 @@ class ManyToManyTable(ManyToOneTable):
             self.col_book_map[existing_item].update(books)
             db.executemany(f'DELETE FROM {self.link_table} WHERE book=? AND {lcol}=?', [
                 (book_id, existing_item) for book_id in books])
-            db.execute('UPDATE {0} SET {1}=? WHERE {1}=?; DELETE FROM {2} WHERE id=?'.format(
-                self.link_table, lcol, table), (existing_item, item_id, item_id))
+            db.rename_category_item(self.name, table, self.link_table, lcol, item_id, existing_item, self.id_map[new_id])
         return affected_books, new_id
 
     def fix_case_duplicates(self, db):
@@ -507,8 +545,11 @@ class ManyToManyTable(ManyToOneTable):
                 done_books = set()
                 main_id = min(v)
                 v.discard(main_id)
+                item_map = {}
                 for item_id in v:
-                    self.id_map.pop(item_id, None)
+                    val = self.id_map.pop(item_id, null)
+                    if val is not null:
+                        item_map[item_id] = val
                     books = self.col_book_map.pop(item_id, set())
                     for book_id in books:
                         if book_id in done_books:
@@ -531,8 +572,7 @@ class ManyToManyTable(ManyToOneTable):
                             db.executemany(
                                 'INSERT INTO {} (book,{}) VALUES (?,?)'.format(self.link_table, self.metadata['link_column']),
                                 tuple((book_id, x) for x in vals))
-                db.executemany('DELETE FROM {} WHERE id=?'.format(self.metadata['table']),
-                    tuple((x,) for x in v))
+                db.delete_category_items(self.name, self.metadata['table'], item_map)
 
 
 class AuthorsTable(ManyToManyTable):
@@ -590,6 +630,7 @@ class AuthorsTable(ManyToManyTable):
 class FormatsTable(ManyToManyTable):
 
     do_clean_on_remove = False
+    supports_notes = False
 
     def read_id_maps(self, db):
         pass
@@ -675,6 +716,8 @@ class FormatsTable(ManyToManyTable):
 
 class IdentifiersTable(ManyToManyTable):
 
+    supports_notes = False
+
     def read_id_maps(self, db):
         pass
 
