@@ -4,19 +4,16 @@
 
 import json
 import os
-import re
 import sys
-import time
 from collections import defaultdict
+from concurrent.futures import ThreadPoolExecutor
 from datetime import datetime
 from functools import partial
 from itertools import count
-from math import ceil
 
 from lxml.etree import Comment
 
 from calibre import detect_ncpus, force_unicode, prepare_string_for_xml
-from calibre.constants import iswindows
 from calibre.customize.ui import plugin_for_input_format
 from calibre.ebooks.oeb.base import EPUB, OEB_DOCS, OEB_STYLES, OPF, SMIL, XHTML, XHTML_NS, XLINK, rewrite_links, urlunquote
 from calibre.ebooks.oeb.base import XPath as _XPath
@@ -25,23 +22,23 @@ from calibre.ebooks.oeb.polish.container import Container as ContainerBase
 from calibre.ebooks.oeb.polish.cover import find_cover_image, find_cover_image_in_page, find_cover_page
 from calibre.ebooks.oeb.polish.toc import from_xpaths, get_landmarks, get_toc
 from calibre.ebooks.oeb.polish.utils import guess_type
-from calibre.ptempfile import PersistentTemporaryDirectory
 from calibre.srv.metadata import encode_datetime
-from calibre.srv.opts import grouper
 from calibre.utils.date import EPOCH
-from calibre.utils.filenames import rmtree
-from calibre.utils.ipc.simple_worker import start_pipe_worker
 from calibre.utils.logging import default_log
-from calibre.utils.serialize import json_dumps, json_loads, msgpack_dumps, msgpack_loads
+from calibre.utils.serialize import json_dumps, json_loads, msgpack_loads
 from calibre.utils.short_uuid import uuid4
-from calibre_extensions import speedup
 from calibre_extensions.fast_css_transform import transform_properties
 from polyglot.binary import as_base64_unicode as encode_component
 from polyglot.binary import from_base64_bytes
 from polyglot.binary import from_base64_unicode as decode_component
-from polyglot.builtins import as_bytes, iteritems
+from polyglot.builtins import as_bytes
 from polyglot.urllib import quote, urlparse
 
+try:
+    from calibre_extensions.speedup import get_num_of_significant_chars
+except ImportError:  # running from source without updated binary
+    def get_num_of_significant_chars(elem):
+        return len(getattr(elem, 'text', '') or '') + len(getattr(elem, 'tail', '') or '')
 RENDER_VERSION = 1
 
 BLANK_JPEG = b'\xff\xd8\xff\xdb\x00C\x00\x03\x02\x02\x02\x02\x02\x03\x02\x02\x02\x03\x03\x03\x03\x04\x06\x04\x04\x04\x04\x04\x08\x06\x06\x05\x06\t\x08\n\n\t\x08\t\t\n\x0c\x0f\x0c\n\x0b\x0e\x0b\t\t\r\x11\r\x0e\x0f\x10\x10\x11\x10\n\x0c\x12\x13\x12\x10\x13\x0f\x10\x10\x10\xff\xc9\x00\x0b\x08\x00\x01\x00\x01\x01\x01\x11\x00\xff\xcc\x00\x06\x00\x10\x10\x05\xff\xda\x00\x08\x01\x01\x00\x00?\x00\xd2\xcf \xff\xd9'  # noqa
@@ -143,37 +140,10 @@ def anchor_map(root):
 
 def get_length(root):
     ans = 0
-
-    fast = getattr(speedup, 'get_element_char_length', None)
-    if fast is None:
-        ignore_tags = frozenset('script style title noscript'.split())
-        img_tags = ('img', 'svg')
-        strip_space = re.compile(r'\s+')
-
-        def count(elem):
-            tag = getattr(elem, 'tag', count)
-            if callable(tag):
-                return len(strip_space.sub('', getattr(elem, 'tail', None) or ''))
-            num = 0
-            tname = tag.rpartition('}')[-1].lower()
-            if elem.text and tname not in ignore_tags:
-                num += len(strip_space.sub('', elem.text))
-            if elem.tail:
-                num += len(strip_space.sub('', elem.tail))
-            if tname in img_tags:
-                num += 1000
-            return num
-    else:
-        def count(elem):
-            tag = getattr(elem, 'tag', count)
-            if callable(tag):
-                return fast('', None, getattr(elem, 'tail', None))
-            return fast(tag, elem.text, elem.tail)
-
     for body in root.iterchildren(XHTML('body')):
-        ans += count(body)
+        ans += get_num_of_significant_chars(body)
         for elem in body.iterdescendants():
-            ans += count(elem)
+            ans += get_num_of_significant_chars(elem)
     return ans
 
 
@@ -193,6 +163,19 @@ def toc_anchor_map(toc):
     return dict(ans)
 
 
+def pagelist_anchor_map(page_list):
+    ans = defaultdict(list)
+    seen_map = defaultdict(set)
+    for i, x in enumerate(page_list):
+        x['id'] = i + 1
+        name = x['dest']
+        frag = x['frag']
+        if name and frag not in seen_map[name]:
+            ans[name].append({'id': x['id'], 'pagenum': x['pagenum'], 'frag':frag})
+            seen_map[name].add(frag)
+    return dict(ans)
+
+
 class SimpleContainer(ContainerBase):
 
     tweak_mode = True
@@ -512,102 +495,6 @@ def transform_html(container, name, virtualize_resources, link_uid, link_to_map,
         f.write(shtml)
 
 
-class RenderManager:
-
-    def __init__(self, max_workers):
-        self.max_workers = max_workers
-
-    def launch_worker(self):
-        with open(os.path.join(self.tdir, f'{len(self.workers)}.json'), 'wb') as output:
-            error = open(os.path.join(self.tdir, f'{len(self.workers)}.error'), 'wb')
-            p = start_pipe_worker('from calibre.srv.render_book import worker_main; worker_main()', stdout=error, stderr=error)
-            p.output_path = output.name
-            p.error_path = error.name
-        self.workers.append(p)
-
-    def __enter__(self):
-        self.workers = []
-        self.tdir = PersistentTemporaryDirectory()
-        return self
-
-    def __exit__(self, *a):
-        while self.workers:
-            p = self.workers.pop()
-            if p.poll() is not None:
-                continue
-            p.terminate()
-            if not iswindows and p.poll() is None:
-                time.sleep(0.02)
-                if p.poll() is None:
-                    p.kill()
-        del self.workers
-        try:
-            rmtree(self.tdir)
-        except OSError:
-            time.sleep(0.1)
-            try:
-                rmtree(self.tdir)
-            except OSError:
-                pass
-        del self.tdir
-
-    def launch_workers(self, names, in_process_container):
-        num_workers = min(detect_ncpus(), len(names))
-        if self.max_workers:
-            num_workers = min(num_workers, self.max_workers)
-        if num_workers > 1:
-            if len(names) < 3 or sum(os.path.getsize(in_process_container.name_path_map[n]) for n in names) < 128 * 1024:
-                num_workers = 1
-        if num_workers > 1:
-            num_other_workers = num_workers - 1
-            while len(self.workers) < num_other_workers:
-                self.launch_worker()
-        return num_workers
-
-    def __call__(self, names, args, in_process_container):
-        num_workers = len(self.workers) + 1
-        if num_workers == 1:
-            return [process_book_files(names, *args, container=in_process_container)]
-
-        group_sz = int(ceil(len(names) / num_workers))
-        groups = tuple(grouper(group_sz, names))
-        for group, worker in zip(groups[:-1], self.workers):
-            worker.stdin.write(as_bytes(msgpack_dumps((worker.output_path, group,) + args)))
-            worker.stdin.flush(), worker.stdin.close()
-            worker.job_sent = True
-
-        for worker in self.workers:
-            if not hasattr(worker, 'job_sent'):
-                worker.stdin.write(b'_'), worker.stdin.flush(), worker.stdin.close()
-
-        error = None
-        results = [process_book_files(groups[-1], *args, container=in_process_container)]
-        for worker in self.workers:
-            if not hasattr(worker, 'job_sent'):
-                worker.wait()
-                continue
-            if worker.wait() != 0:
-                with open(worker.error_path, 'rb') as f:
-                    error = f.read().decode('utf-8', 'replace')
-            else:
-                with open(worker.output_path, 'rb') as f:
-                    results.append(msgpack_loads(f.read()))
-        if error is not None:
-            raise Exception('Render worker failed with error:\n' + error)
-        return results
-
-
-def worker_main():
-    stdin = getattr(sys.stdin, 'buffer', sys.stdin)
-    raw = stdin.read()
-    if raw == b'_':
-        return
-    args = msgpack_loads(raw)
-    result = process_book_files(*args[1:])
-    with open(args[0], 'wb') as f:
-        f.write(as_bytes(msgpack_dumps(result)))
-
-
 def virtualize_html(container, name, link_uid, link_to_map, virtualized_names):
 
     changed = set()
@@ -645,7 +532,7 @@ def virtualize_html(container, name, link_uid, link_to_map, virtualized_names):
 __smil_file_names__ = ''
 
 
-def process_book_files(names, container_dir, opfpath, virtualize_resources, link_uid, data_for_clone, container=None):
+def process_book_files(names, container_dir, opfpath, virtualize_resources, link_uid, data_for_clone=None, container=None):
     if container is None:
         container = SimpleContainer(container_dir, opfpath, default_log, clone_data=data_for_clone)
         container.cloned = False
@@ -675,9 +562,19 @@ def process_book_files(names, container_dir, opfpath, virtualize_resources, link
     return link_to_map, html_data, virtualized_names, smil_map
 
 
+def calculate_number_of_workers(names, in_process_container, max_workers):
+    num_workers = min(detect_ncpus(), len(names))
+    if max_workers:
+        num_workers = min(num_workers, max_workers)
+    if num_workers > 1:
+        if len(names) < 3 or sum(os.path.getsize(in_process_container.name_path_map[n]) for n in names) < 128 * 1024:
+            num_workers = 1
+    return num_workers
+
+
 def process_exploded_book(
-    book_fmt, opfpath, input_fmt, tdir, render_manager, log=None, book_hash=None, save_bookmark_data=False,
-    book_metadata=None, virtualize_resources=True
+    book_fmt, opfpath, input_fmt, tdir, log=None, book_hash=None, save_bookmark_data=False,
+    book_metadata=None, virtualize_resources=True, max_workers=1
 ):
     log = log or default_log
     container = SimpleContainer(tdir, opfpath, log)
@@ -687,15 +584,6 @@ def process_exploded_book(
     def needs_work(mt):
         return mt in OEB_STYLES or mt in OEB_DOCS or mt in ('image/svg+xml', 'application/smil', 'application/smil+xml')
 
-    def work_priority(name):
-        # ensure workers with large files or stylesheets
-        # have the less names
-        size = os.path.getsize(container.name_path_map[name]),
-        is_html = container.mime_map.get(name) in OEB_DOCS
-        return (0 if is_html else 1), size
-
-    if not is_comic:
-        render_manager.launch_workers(tuple(n for n, mt in iteritems(container.mime_map) if needs_work(mt)), container)
 
     bookmark_data = None
     if save_bookmark_data:
@@ -708,12 +596,14 @@ def process_exploded_book(
     # browser has no good way to distinguish between zero byte files and
     # load failures.
     excluded_names = {
-        name for name, mt in iteritems(container.mime_map) if
+        name for name, mt in container.mime_map.items() if
         name == container.opf_name or mt == guess_type('a.ncx') or name.startswith('META-INF/') or
         name == 'mimetype' or not container.has_name_and_is_not_empty(name)}
     raster_cover_name, titlepage_name = create_cover_page(container, input_fmt.lower(), is_comic, book_metadata)
 
-    toc = get_toc(container, verify_destinations=False).to_dict(count())
+    tocobj = get_toc(container, verify_destinations=False)
+    page_list = tocobj.page_list or []
+    toc = tocobj.to_dict(count())
     if not toc or not toc.get('children'):
         toc = from_xpaths(container, ['//h:h1', '//h:h2', '//h:h3']).to_dict(count())
     spine = [name for name, is_linear in container.spine_names]
@@ -746,23 +636,29 @@ def process_exploded_book(
         'landmarks': landmarks,
         'link_to_map': {},
         'page_progression_direction': page_progression_direction,
+        'page_list': page_list,
+        'page_list_anchor_map': pagelist_anchor_map(page_list),
     }
 
-    names = sorted(
-        (n for n, mt in iteritems(container.mime_map) if needs_work(mt)),
-        key=work_priority)
+    names_that_need_work = tuple(n for n, mt in container.mime_map.items() if needs_work(mt))
+    num_workers = calculate_number_of_workers(names_that_need_work, container, max_workers)
+    results = []
+    if num_workers < 2:
+        results.append(process_book_files(names_that_need_work, tdir, opfpath, virtualize_resources, book_render_data['link_uid'], container=container))
+    else:
+        with ThreadPoolExecutor(max_workers=num_workers) as executor:
+            futures = tuple(
+                executor.submit(process_book_files, (name,), tdir, opfpath, virtualize_resources, book_render_data['link_uid'], container=container)
+                for name in names_that_need_work)
+            for future in futures:
+                results.append(future.result())
 
-    results = render_manager(
-        names, (
-            tdir, opfpath, virtualize_resources, book_render_data['link_uid'], container.data_for_clone()
-        ), container
-    )
     ltm = book_render_data['link_to_map']
     html_data = {}
     virtualized_names = set()
 
     def merge_ltm(dest, src):
-        for k, v in iteritems(src):
+        for k, v in src.items():
             if k in dest:
                 dest[k] |= v
             else:
@@ -783,7 +679,7 @@ def process_exploded_book(
         html_data.update(hdata)
         virtualized_names |= vnames
         merge_smil_map(smil_map)
-        for k, v in iteritems(link_to_map):
+        for k, v in link_to_map.items():
             if k in ltm:
                 merge_ltm(ltm[k], v)
             else:
@@ -820,8 +716,8 @@ def process_exploded_book(
         os.remove(container.name_path_map[name])
 
     ltm = book_render_data['link_to_map']
-    for name, amap in iteritems(ltm):
-        for k, v in tuple(iteritems(amap)):
+    for name, amap in ltm.items():
+        for k, v in tuple(amap.items()):
             amap[k] = tuple(v)  # needed for JSON serialization
 
     data = as_bytes(json.dumps(book_render_data, ensure_ascii=False))
@@ -904,41 +800,40 @@ def get_stored_annotations(container, bookmark_data):
                 yield {'type': 'last-read', 'pos': epubcfi, 'pos_type': 'epubcfi', 'timestamp': EPOCH}
 
 
-def render(pathtoebook, output_dir, book_hash=None, serialize_metadata=False, extract_annotations=False, virtualize_resources=True, max_workers=1):
+def render(pathtoebook, output_dir, book_hash=None, serialize_metadata=False, extract_annotations=False, virtualize_resources=True, max_workers=0):
     pathtoebook = os.path.abspath(pathtoebook)
-    with RenderManager(max_workers) as render_manager:
-        mi = None
-        if serialize_metadata:
-            from calibre.customize.ui import quick_metadata
-            from calibre.ebooks.metadata.meta import get_metadata
-            with open(pathtoebook, 'rb') as f, quick_metadata:
-                mi = get_metadata(f, os.path.splitext(pathtoebook)[1][1:].lower())
-        book_fmt, opfpath, input_fmt = extract_book(pathtoebook, output_dir, log=default_log)
-        container, bookmark_data = process_exploded_book(
-            book_fmt, opfpath, input_fmt, output_dir, render_manager,
-            book_hash=book_hash, save_bookmark_data=extract_annotations,
-            book_metadata=mi, virtualize_resources=virtualize_resources
-        )
-        if serialize_metadata:
-            from calibre.ebooks.metadata.book.serialize import metadata_as_dict
-            d = metadata_as_dict(mi)
-            d.pop('cover_data', None)
-            serialize_datetimes(d), serialize_datetimes(d.get('user_metadata', {}))
-            with open(os.path.join(output_dir, 'calibre-book-metadata.json'), 'wb') as f:
-                f.write(json_dumps(d))
-        if extract_annotations:
-            annotations = None
-            if bookmark_data:
-                annotations = json_dumps(tuple(get_stored_annotations(container, bookmark_data)))
-            if annotations:
-                with open(os.path.join(output_dir, 'calibre-book-annotations.json'), 'wb') as f:
-                    f.write(annotations)
+    mi = None
+    if serialize_metadata:
+        from calibre.customize.ui import quick_metadata
+        from calibre.ebooks.metadata.meta import get_metadata
+        with open(pathtoebook, 'rb') as f, quick_metadata:
+            mi = get_metadata(f, os.path.splitext(pathtoebook)[1][1:].lower())
+    book_fmt, opfpath, input_fmt = extract_book(pathtoebook, output_dir, log=default_log)
+    container, bookmark_data = process_exploded_book(
+        book_fmt, opfpath, input_fmt, output_dir, max_workers=max_workers,
+        book_hash=book_hash, save_bookmark_data=extract_annotations,
+        book_metadata=mi, virtualize_resources=virtualize_resources
+    )
+    if serialize_metadata:
+        from calibre.ebooks.metadata.book.serialize import metadata_as_dict
+        d = metadata_as_dict(mi)
+        d.pop('cover_data', None)
+        serialize_datetimes(d), serialize_datetimes(d.get('user_metadata', {}))
+        with open(os.path.join(output_dir, 'calibre-book-metadata.json'), 'wb') as f:
+            f.write(json_dumps(d))
+    if extract_annotations:
+        annotations = None
+        if bookmark_data:
+            annotations = json_dumps(tuple(get_stored_annotations(container, bookmark_data)))
+        if annotations:
+            with open(os.path.join(output_dir, 'calibre-book-annotations.json'), 'wb') as f:
+                f.write(annotations)
 
 
 def render_for_viewer(path, out_dir, book_hash):
     return render(
         path, out_dir, book_hash=book_hash, serialize_metadata=True,
-        extract_annotations=True, virtualize_resources=False, max_workers=0
+        extract_annotations=True, virtualize_resources=False
     )
 
 
